;-----------------------------------------------------------------------------
; RetroForth for FreeBSD
;-----------------------------------------------------------------------------
include "../rx/macros"

macro syscall_system A {
   push 0x0
    mov eax, A
    int 0x80
    setc dl
    add esp, 4
}

format elf executable
entry _start

_start:
        pop dword [arg_count]
        pop dword [args]

        call bootstrap_rx
	upsh boot_freebsd
	upsh [boot_freebsd.size]
	call eval

	jmp boot


arg_count dd 0
args      dd 0
code 'cmdline', _cmdline
        upsh [args]
next
code '#args', _num_args
        upsh [arg_count]
next



;---------------------------------------------------------------------
; BSD syscall interface
; Description:
; syscall ( args argcount syscall# -- result flag )
; args: as few as 0 or as many as 6 arguments
; argcount: the number of arguments
; syscall-number: a number from /usr/include/asm/unistd.h
; This calling convention was borrowed from Marcel Hendrix's port of 
; eforth for Linux.

code 'bsd-syscall', _syscall2
        upop edx                ; Save the system call #
.syscall_1:
        cmp eax, 1              ; Is this a one argument system call?
        jnz .syscall_2          ;
        drop                    ; Forget the count
        push esi                ; Save ESI
        push eax                ; Argument #1
        syscall_system edx              ; Do the system call (via macro)
        mov ebx, eax            ; Temporarily save result
        add esp, 4              ; Clean up return stack
        pop esi                 ; Restore ESI
        mov eax, ebx            ; Put the result in EAX
        add esi, 4
        jmp .syscall_exit       ;
.syscall_2:                     ; Is this a two argument system call?
        cmp eax, 2              ;
        jnz .syscall_3          ;
        drop                    ;
        push esi                ; Preserve ESI
        push eax                ; Argument #1
        drop                    ;
        push eax                ; Argument #2
        syscall_system edx              ; Do the system call
        mov ebx, eax            ; Save the result
        add esp, 2*4            ;
        pop esi                 ; Restore ESI
        mov eax, ebx            ; Put the result on TOS
        add esi, 4
        jmp .syscall_exit       ;
.syscall_3:                     ; Is this a three argument system call?
        cmp eax, 3              ;
        jnz .syscall_4          ;
        drop                    ;
        push esi                ;
        push eax                ; Argument #1
        drop                    ;
        push eax                ; Argument #2
        drop                    ;
        push eax                ; Argument #3
        syscall_system edx              ; Do the system call
        mov ebx, eax            ;
        add esp, 3*4            ;
        pop esi                 ;
        mov eax, ebx            ;
        add esi, 4
        jmp .syscall_exit       ;
.syscall_4:                     ;
        cmp eax, 4              ; Is this a four argument system call?
        jnz .syscall_5          ;
        drop                    ;
        push esi                ;
        push eax                ; Argument #1
        drop                    ;
        push eax                ; Argument #2
        drop                    ;
        push eax                ; Argument #3
        drop                    ;
        push eax                ; Argument #4
        syscall_system edx              ; Do the system call
        mov ebx, eax            ;
        add esp, 4*4            ;
        pop esi                 ;
        mov eax, ebx            ;
        add esi, 4
        jmp .syscall_exit       ;
.syscall_5:                     ; Five arguments?
        cmp eax, 5              ;
        jnz .syscall_6          ;
        drop                    ;
        push esi                ;
        push eax                ; Argument #1
        drop                    ;
        push eax                ; Argument #2
        drop                    ;
        push eax                ; Argument #3
        drop                    ;
        push eax                ; Argument #4
        drop                    ;
        push eax                ; Argument #5
        syscall_system edx              ; Do the system call
        mov ebx, eax            ;
        add esp, 5*4            ;
        pop esi                 ;
        dup                     ;
        mov eax, ebx            ;
        add esi, 4
        jmp .syscall_exit       ;
.syscall_6:                     ; Six arguments?
        cmp eax, 6              ;
        jnz .syscall_exit       ; If not between 1-6, exit
        drop                    ;
        push esi                ;
        push eax                ; Argument #1
        drop                    ;
        push eax                ; Argument #2
        drop                    ;
        push eax                ; Argument #3
        drop                    ;
        push eax                ; Argument #4
        drop                    ;
        push eax                ; Argument #5
        drop                    ;
        push eax                ; Argument #6
        syscall_system edx              ; Do the system call
        mov ebx, eax            ;
        add esp, 6*4            ;
        pop esi                 ;
        dup                     ;
        mov eax, ebx            ;
        add esi, 4
.syscall_exit:
	xchg eax, [esi]
	movzx   eax,dl
next
;---------------------------------------------------------------------
boot_freebsd:
	include "../modules.inc"
        file "freebsd.f"
.size dd $-boot_freebsd
;---------------------------------------------------------------------
code 'boot', boot
next      
;---------------------------------------------------------------------
include "../rx/rx"

